import "Turbine.UI"; 
import "Turbine.UI.Lotro"; 
import "Turbine.Gameplay";

-- setup variables
localPlayer = Turbine.Gameplay.LocalPlayer:GetInstance();
currentWallet = localPlayer:GetWallet();
currentCharacterName = localPlayer:GetName();
currencyList = {};

-------------------------------------------------------------------------------------------------


function GetWalletItemByName(ItemName)
	
	for i = 1, currentWallet:GetSize() do
	
		local walletItem = currentWallet:GetItem(i);
		
		if ItemName == walletItem:GetName() then
		
			return walletItem;

		end
		
	end
	
	return nil;
	
end


function GetItemFromArrayByName(ArrayName, ItemName)

	for i, ArrayItem in ipairs(ArrayName) do
	
		if ArrayItem.Name == ItemName then
			
			return ArrayItem;
		
		end
		
	end
	
	return nil;

end


-------------------------------------------------------------------------------------------------


function AddCallback(object, event, callback)
    if (object[event] == nil) then
        object[event] = callback;
    else
        if (type(object[event]) == "table") then
            table.insert(object[event], callback);
        else
            object[event] = {object[event], callback};
        end
    end
    return callback;
end
function RemoveCallback(object, event, callback)
    if (object[event] == callback) then
        object[event] = nil;
    else
        if (type(object[event]) == "table") then
            local size = table.getn(object[event]);
            for i = 1, size do
                if (object[event][i] == callback) then
                    table.remove(object[event], i);
                    break;
                end
            end
        end
    end
end


-------------------------------------------------------------------------------------------------


function UpdateCurrencyList()

	local ChangesMade = false;
	
--	-- remove items from currencyList that are no longer in the wallet
--	-- this part needs to move to seperate function, since I'm having to initialize piecemeal in "Update"
--	for i, currencyItem in ipairs(currencyList) do
--	
--		-- check if this is still in the wallet
--		local walletItem = GetWalletItemByName(currencyItem.Name);
--	
--		if walletItem == nil then -- item no longer in wallet, so remove from list
--		
--			table.remove(currencyList, i);
--			ChangesMade = true;
--			
--		end
--
--	end
	
	-- add new items if they're not in currencyList, but are in wallet
	for WalletIdx = 1, currentWallet:GetSize() do
	
		local walletItem = currentWallet:GetItem(WalletIdx);

		if walletItem:IsAccountItem() then
		
			local walletItemName = walletItem:GetName();
			
			-- check if this is already in the list or not
			local currencyItem = GetItemFromArrayByName(currencyList, walletItemName);
			
			if currencyItem == nil then    -- we need to add this currency to our list
			
				local Assignment = {};
				
				Assignment.Name = currentCharacterName;
				Assignment.Amount = walletItem:GetQuantity();  
				-- I'd prefer to initialize Amount to zero, but due to Update workaround,
				-- this has to work for earning new currencies in-game, in addition to initialization,
				-- so it all gets assigned to the first character who sees it.  Players will need to
				-- manually re-allocate on their first use of the plugin, anyway.

				local CharacterAssignments = {};
				
				table.insert(CharacterAssignments, Assignment);
			
				currencyItem = {};
				
				currencyItem.Name = walletItemName;
				currencyItem.Assignments = CharacterAssignments;
				
				table.insert(currencyList, currencyItem);
			
				ChangesMade = true;
				
			end
			
		end
		
	end
	
	-- ensure current character is included on all items in currencyList
	for i, currencyItem in ipairs(currencyList) do
	
		local CharacterAssignments = currencyItem.Assignments;
		
		local Assignment = GetItemFromArrayByName(CharacterAssignments, currentCharacterName);
		
		if Assignment == nil then
		
			Assignment = {};
			
			Assignment.Name = currentCharacterName;
			Assignment.Amount = 0;
			
			table.insert(CharacterAssignments, Assignment);
			
			currencyItem.Assignments = CharacterAssignments;
			
			ChangesMade = true;
		
		end
	
	end
	
	-- inform user of changes
	if ChangesMade then
	
		Turbine.Shell.WriteLine("PerCharWallet: table of wallet currencies updated");
		
	end
	
end


function AssignCurrency(CharName, CurrencyName, AddedAmount)

	local CurrencyFound = false;
	local AssignmentFound = false;
	
	for i, currencyItem in ipairs(currencyList) do
	
		if currencyItem.Name == CurrencyName then
		
			CurrencyFound = i;
		
			for j, Assignment in ipairs(currencyItem.Assignments) do
			
				if Assignment.Name == CharName then
				
					AssignmentFound = j;
				
					currencyList[i].Assignments[j].Amount = currencyList[i].Assignments[j].Amount + AddedAmount;
				
				end
			
			end
		
		end
	
	end
	
	if not AssignmentFound then

		local Assignment = {};
		
		Assignment.Name = currentCharacterName;
		Assignment.Amount = AddedAmount;
		
		if CurrencyFound then
		
			table.insert(currencyList[CurrencyFound].Assignments, Assignment);
		
		else
		
			local CharacterAssignments = {};
			
			table.insert(CharacterAssignments, Assignment);
		
			local currencyItem = {};
			
			currencyItem.Name = CurrencyName;
			currencyItem.Assignments = CharacterAssignments;
			
			table.insert(currencyList, currencyItem);
		
		end
	
	end

end


-------------------------------------------------------------------------------------------------


-- register and handle shell commands
PerCharWallet = Turbine.ShellCommand();
Turbine.Shell.AddCommand("PerCharWallet;pcw", PerCharWallet);


function PerCharWallet:GetHelp()

	Turbine.Shell.WriteLine("");
	Turbine.Shell.WriteLine("Per Character Wallet v" .. Plugins["PerCharWallet"]:GetVersion() .. " by Niwashi.  usage:");
	Turbine.Shell.WriteLine("You can open the main Per Character Wallet window by typing either of the following two commands into the chat window:");
	Turbine.Shell.WriteLine("/PerCharWallet");
	Turbine.Shell.WriteLine("/pcw");
	Turbine.Shell.WriteLine("(Following either of those commands with the word \"help\" will instead bring up this help description.)");
	Turbine.Shell.WriteLine("The main Per Character Wallet window displays the account-wide currencies allocated to your current character (as well as the account-wide total).");
	Turbine.Shell.WriteLine("When you earn or spend any account-wide currencies, those will automatically be allocated to/from the character who earned/spent them,");
	Turbine.Shell.WriteLine("but you can manually re-assign them to any of your other characters by clicking the \"Assignments\" button on the main window");
	Turbine.Shell.WriteLine("and selecting the currency from the drop-down list.  Use the Up/Down arrow buttons to unassign currency from one character, and then assign it to another.");
	Turbine.Shell.WriteLine("There are four sets of those Up/Down arrow buttons for each character.  This is to let you assign them in increments of 1000, 100, 10, or 1.");
	Turbine.Shell.WriteLine("");
	
end


function PerCharWallet:GetShortHelp()

	return('PerCharWallet - usage: /PerCharWallet show|list|setup|update  (can also use "/pcw show|list|setup|update")');
	
end


function PerCharWallet:Execute(cmd, args)

	if (args == "help") then
	
		PerCharWallet:GetHelp();
		
	else
	
		pcwDisplayWindow:SetVisible(true);
		
	end
	
end


-------------------------------------------------------------------------------------------------

QCCallbackHandlers = {};

-- handle saving and loading
function saveData()

	Turbine.PluginData.Save(Turbine.DataScope.Server, "PerCharWallet_CurrencyList", currencyList);

end

--Turbine.Plugin.Unload = function (sender, args)
--	saveData();
--end
function UnloadMe()

	-- release QuantityChanged callbacks
	for walletItem, callbackHandler in pairs(QCCallbackHandlers) do

		RemoveCallback(walletItem, "QuantityChanged", callbackHandler);

	end

	-- save any data that needs saving
	saveData();

end

function loadData()

	function GetCurrencyList()
	
		currencyList = Turbine.PluginData.Load(Turbine.DataScope.Server, "PerCharWallet_CurrencyList");
		
	end

	if pcall(GetCurrencyList) then
		-- no errors
		currencyList = Turbine.PluginData.Load(Turbine.DataScope.Server, "PerCharWallet_CurrencyList");
		
	else
		-- errors
		currencyList = {};
		Turbine.Shell.WriteLine("PerCharWallet: error loading currency list.");
		
	end
	
	if type(currencyList) ~= 'table' then
	
		currencyList = {};
		
	end

end

loadData();


-------------------------------------------------------------------------------------------------


import "Niwashi.PerCharWallet.DisplayWindow";
import "Niwashi.PerCharWallet.SetupWindow";


-- finish
Turbine.Shell.WriteLine("Per Character Wallet v" .. Plugins["PerCharWallet"]:GetVersion() .. " by Niwashi loaded.");
